// uptime.cpp : Defines the entry point for the application.
//

#include "stdafx.h"

HWND ghMainDlg = NULL;
HQUERY ghQuery = NULL;
HCOUNTER ghCounter = NULL;
WINDOWINFO gWi = {0};
COLORREF gColor = 0x00FFFFFF;
POINT gPosition = {0, 0};
POINT gPercent = {0, 0};
NOTIFYICONDATA gShellIcon = {0};
HINSTANCE ghInst = NULL;
BOOL gbDisplayTime = FALSE;

INT APIENTRY WinMain(HINSTANCE hInstance,
                     HINSTANCE hPrevInstance,
                     LPSTR     lpCmdLine,
                     INT       iCmdShow)
{
    INT iResult = ERROR_SUCCESS;
    HANDLE hProcessMutex = NULL;

    __try
    {
        __try
        {
            GetCommandOpts(lpCmdLine);

            ghInst = hInstance;

            hProcessMutex = CreateMutex(NULL, TRUE, "Uptime.mtx");

            if (!hProcessMutex || GetLastError() != ERROR_SUCCESS)
            {
                iResult = GetLastError();
                __leave;
            }

            if (OpenPdh() != ERROR_SUCCESS)
            {
                iResult = GetLastError();
                __leave;
            }

            if (!CreateDialog(hInstance, MAKEINTRESOURCE(IDD_DIALOG_MAIN), NULL, MainDlgProc))
            {
                iResult = GetLastError();
                __leave;
            }

            MSG msg;

            SetProcessWorkingSetSize(GetCurrentProcess(), -1, -1);

            while(GetMessage(&msg, NULL, 0, 0))
            { 
                if(!IsDialogMessage(ghMainDlg, &msg))  
                {
                    TranslateMessage(&msg); 
                    DispatchMessage(&msg); 
                }
            }
        }
        __except(EXCEPTION_EXECUTE_HANDLER)
        {
        }
    }
    __finally
    {
        if (hProcessMutex)
            CloseHandle(hProcessMutex);

        ClosePdh();
    }

    return 0;
}

INT OpenPdh()
{
    CHAR szBuf[MAX_PATH];

    DWORD dwSize = sizeof(szBuf);
    PDH_COUNTER_PATH_ELEMENTS pe = {0};

    pe.szObjectName = "System";
    pe.szCounterName = "System Up Time";

    if (PdhOpenQuery (NULL, 0, &ghQuery) != ERROR_SUCCESS ||
        PdhMakeCounterPath(&pe, &szBuf[0], &dwSize, 0) != ERROR_SUCCESS ||
        PdhAddCounter(ghQuery, &szBuf[0], 0, &ghCounter) != ERROR_SUCCESS)
        return GetLastError();

    return ERROR_SUCCESS;

}

VOID ClosePdh()
{
    if (ghCounter)
        PdhRemoveCounter(ghCounter);

    if (ghQuery)
        PdhCloseQuery(ghQuery);
}

BOOL CALLBACK MainDlgProc(HWND hDlg, UINT uiMsg, WPARAM wParam, LPARAM lParam)
{
    switch(uiMsg) 
    {
        case WM_TIMER:
            return OnTimer(hDlg, uiMsg, wParam, lParam);

        case WM_CTLCOLORSTATIC:
            return OnCtlColorStatic(hDlg, uiMsg, wParam, lParam);

        case WM_TRAYNOTIFY:
            if (lParam == WM_RBUTTONDOWN)
                PostMessage(hDlg, WM_CLOSE, 0, 0);
            return TRUE;

        case WM_SYSCOLORCHANGE:
            return OnSysColorChange(hDlg, uiMsg, wParam, lParam);

        case WM_DISPLAYCHANGE:
            return OnDisplayChange(hDlg, uiMsg, wParam, lParam);

        /*
        case WM_COMMAND:
            return OnCommand(hDlg, uiMsg, wParam, lParam);

        case WM_SYSCOMMAND:
            return OnSysCommand(hDlg, uiMsg, wParam, lParam);
        */

        case WM_INITDIALOG:
            return OnInitDialog(hDlg, uiMsg, wParam, lParam);

        case WM_CLOSE:
            return OnClose(hDlg, uiMsg, wParam, lParam);

        case WM_DESTROY:
            return OnDestroy(hDlg, uiMsg, wParam, lParam);

        default:
            break;
    }

    return FALSE; // Didn't handled message
}

BOOL OnCommand(HWND hDlg, UINT uiMsg, WPARAM wParam, LPARAM lParam)
{
    BOOL bHandled = FALSE;

    switch(LOWORD(wParam))
    {
        case 0:
        default:
            break;

    }

    SetWindowLong(hDlg, DWL_MSGRESULT, (bHandled == FALSE));
    return bHandled;
}

BOOL OnDisplayChange(HWND hDlg, UINT uiMsg, WPARAM wParam, LPARAM lParam)
{
    gPosition.x = (LONG) ((float) LOWORD(lParam) * ((float) gPercent.x / 100.0));
    gPosition.y = (LONG) ((float) HIWORD(lParam) * ((float) gPercent.y / 100.0));

    SetWindowPos(ghMainDlg, HWND_BOTTOM, gPosition.x, gPosition.y, 0, 0, SWP_SHOWWINDOW | SWP_NOSIZE | SWP_NOACTIVATE);
    return TRUE;
}

// WM_COMMAND msg handler
BOOL OnSysCommand(HWND hDlg, UINT uiMsg, WPARAM wParam, LPARAM lParam)
{
    BOOL bHandled = FALSE;

    switch(LOWORD(wParam))
    {
        case SC_CLOSE:
            break;

        default:
            break;

    }

    SetWindowLong(hDlg, DWL_MSGRESULT, (bHandled == FALSE));
    return bHandled;
}

// WM_CLOSE msg handler
BOOL OnClose(HWND hDlg, UINT uiMsg, WPARAM wParam, LPARAM lParam)
{
    DestroyWindow(hDlg);
    SetWindowLong(hDlg, DWL_MSGRESULT, 0);
    return TRUE;
}

// WM_DESTROY msg handler
BOOL OnDestroy(HWND hDlg, UINT uiMsg, WPARAM wParam, LPARAM lParam)
{
    KillTimer(hDlg, 0);
    PostQuitMessage(0);
    SetWindowLong(hDlg, DWL_MSGRESULT, 0);
    return TRUE;
}

// WM_INITDIALOG msg handler
BOOL OnInitDialog(HWND hDlg, UINT uiMsg, WPARAM wParam, LPARAM lParam)
{
    ghMainDlg = hDlg;
   
    OnTimer(hDlg, 0, 0, 0);
    SetTimer(hDlg, 0, 1000, NULL);

    gWi.cbSize = sizeof(gWi);

    GetWindowInfo(GetDesktopWindow(), &gWi);

    gPosition.x = (LONG) ((float) gWi.rcWindow.right * ((float) gPercent.x / 100.0));
    gPosition.y = (LONG) ((float) gWi.rcWindow.bottom * ((float) gPercent.y / 100.0));

    GetWindowInfo(hDlg, &gWi);

    RECT Rect;

    GetWindowRect(GetDlgItem(hDlg, IDC_STATIC_UPTIME), &Rect);
    MapWindowPoints(NULL, hDlg, (POINT*) &Rect, 2);
    Rect.left += gWi.rcClient.left - gWi.rcWindow.left;
    Rect.top += gWi.rcClient.top - gWi.rcWindow.top;
    Rect.right += gWi.rcClient.left - gWi.rcWindow.left;
    Rect.bottom += gWi.rcClient.top - gWi.rcWindow.top;

    SetWindowRgn(
                    hDlg, 
                    CreateRectRgn(
                                    Rect.left,
                                    Rect.top,
                                    Rect.right,
                                    Rect.bottom
                                 ),
                    FALSE
                );

    SetWindowLong(hDlg, GWL_EXSTYLE, GetWindowLong(hDlg, GWL_EXSTYLE) | WS_EX_LAYERED);
    SetLayeredWindowAttributes(hDlg, GetSysColor(COLOR_3DFACE), NULL, LWA_COLORKEY);

    if (gShellIcon.cbSize)
    {
        gShellIcon.hIcon = (HICON) LoadImage(ghInst, MAKEINTRESOURCE(IDI_ICON1), IMAGE_ICON, 16, 16, LR_DEFAULTCOLOR);
        gShellIcon.hWnd = hDlg;
        gShellIcon.uFlags = NIF_ICON | NIF_MESSAGE;
        gShellIcon.uCallbackMessage = WM_TRAYNOTIFY;

        Shell_NotifyIcon(NIM_ADD, &gShellIcon);
    }

    SetWindowPos(ghMainDlg, HWND_BOTTOM, gPosition.x, gPosition.y, 0, 0, SWP_SHOWWINDOW | SWP_NOSIZE | SWP_NOACTIVATE);

    return FALSE;
}

BOOL OnSysColorChange(HWND hDlg, UINT uiMsg, WPARAM wParam, LPARAM lParam)
{
    SetLayeredWindowAttributes(hDlg, GetSysColor(COLOR_3DFACE), NULL, LWA_COLORKEY);
    SetWindowPos(hDlg, HWND_BOTTOM, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE | SWP_NOACTIVATE);
    return TRUE;
}

BOOL OnCtlColorStatic(HWND hDlg, UINT uiMsg, WPARAM wParam, LPARAM lParam)
{
    HBRUSH hBrush = (HBRUSH) GetStockObject(DC_BRUSH);

    SetDCBrushColor((HDC) wParam, GetSysColor(COLOR_3DFACE));
    SetBkMode((HDC) wParam, TRANSPARENT);

    //0x00bbggrr
    SetTextColor((HDC) wParam, gColor);

    return (BOOL) hBrush;
}

// WM_TIMER msg handler
BOOL OnTimer(HWND hDlg, UINT uiMsg, WPARAM wParam, LPARAM lParam)
{
    CHAR szBuf[128];
    PDH_FMT_COUNTERVALUE pcv;

    szBuf[0] = 0;

    if (PdhCollectQueryData(ghQuery) == ERROR_SUCCESS &&
        PdhGetFormattedCounterValue(ghCounter, PDH_FMT_LARGE , NULL, &pcv) == ERROR_SUCCESS)
    {
        SYSTEMTIME st;

        if (gbDisplayTime)
        {
            GetLocalTime(&st);

            PCHAR pszTemp = "PM";

            if (st.wHour < 12)
            {
                pszTemp = "AM";
                if (!st.wHour)
                    st.wHour = 12;
            }
            else if (st.wHour > 12)
            {
                st.wHour -= 12;
            }

            wsprintf(
                        szBuf, 
                        "%u:%02u:%02u:%02u - %u:%02u:%02u %s, %u/%u/%u",
                        (UINT) (pcv.largeValue / (3600 * 24)),
                        (UINT) (pcv.largeValue % (3600 * 24) / 3600),
                        (UINT) ((pcv.largeValue % 3600) / 60),
                        (UINT) (pcv.largeValue % 60),
                        st.wHour,
                        st.wMinute,
                        st.wSecond,
                        pszTemp,
                        st.wMonth,
                        st.wDay,
                        st.wYear
                    );
        }
        else
        {
            wsprintf(
                        szBuf, 
                        "%u:%02u:%02u:%02u",
                        (UINT) (pcv.largeValue / (3600 * 24)),
                        (UINT) (pcv.largeValue % (3600 * 24) / 3600),
                        (UINT) ((pcv.largeValue % 3600) / 60),
                        (UINT) (pcv.largeValue % 60)
                    );
        }
    }

    SetDlgItemText(hDlg, IDC_STATIC_UPTIME, &szBuf[0]);
    
    return TRUE;
}

VOID GetCommandOpts(CHAR * pszCmdOpts)
{
    CHAR *pszTemp;

    _strlwr(pszCmdOpts);

    pszTemp = strstr(pszCmdOpts, "-?");

    if (pszTemp)
    {
        MessageBox(NULL, UPTIME_USAGE, "Uptime Usage", MB_ICONINFORMATION);
        ExitProcess(~0);
    }

    pszTemp = strstr(pszCmdOpts, "-x");

    if (pszTemp)
    {
        pszTemp += 2;
        gPercent.x = strtoul(pszTemp, NULL, 0);
        if (gPercent.x > 100)
            gPercent.x = 0;
    }

    pszTemp = strstr(pszCmdOpts, "-y");

    if (pszTemp)
    {
        pszTemp += 2;
        gPercent.y = strtoul(pszTemp, NULL, 0);
        if (gPercent.y > 100)
            gPercent.y = 0;
    }

    pszTemp = strstr(pszCmdOpts, "-c");

    if (pszTemp)
    {
        pszTemp += 2;
        gColor = strtoul(pszTemp, NULL, 16);
    }

    pszTemp = strstr(pszCmdOpts, "-t");

    if (pszTemp)
        gShellIcon.cbSize = NOTIFYICONDATA_V1_SIZE;

    pszTemp = strstr(pszCmdOpts, "-d");

    if (pszTemp)
        gbDisplayTime = TRUE;
}